(*
 * Copyright 2021 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *)

[inherit('common')]
module pJSON;

type
   cJSON_ptr = integer_address;

[external('CJSON_PARSE')]
function cJSON_Parse(%IMMED jsonstr : c_str_t) : cJSON_ptr; extern;

function pJSON_Parse(jsonstr : pstr) : cJSON_Ptr;

var
   xjsonstr : c_str_t;

begin
   xjsonstr := malloc_c_str(jsonstr);
   pJSON_Parse := cJSON_Parse(xjsonstr);
   free_c_str(xjsonstr);
end;

[external('CJSON_GETARRAYSIZE')]
function cJSON_GetArraySize(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_GetArraySize(json : cJSON_ptr) : integer;

begin
   pJSON_GetArraySize := cJSON_GetArraySize(json);
end;

[external('CJSON_GETARRAYITEM')]
function cJSON_GetArrayItem(%IMMED json : cJSON_ptr; %IMMED ix : integer) : cJSON_ptr; extern;

function pJSON_GetArrayItem(json : cJSON_ptr; ix : integer) : cJSON_ptr;

begin
   pJSON_GetArrayItem := cJSON_GetArrayItem(json, ix);
end;

[external('CJSON_GETOBJECTITEM')]
function cJSON_GetObjectItem(%IMMED json : cJSON_ptr; %IMMED prop : c_str_t) : cJSON_ptr; extern;

function pJSON_GetObjectItem(json : cJSON_ptr; prop : pstr) : cJSON_ptr;

var
   xprop : c_str_t;

begin
   xprop := malloc_c_str(prop);
   pJSON_GetObjectItem := cJSON_GetObjectItem(json, xprop);
   free_c_str(xprop);
end;

[external('CJSON_PRINTUNFORMATTED')]
function cJSON_PrintUnformatted(%IMMED json : cJSON_ptr) : c_str_t; extern;

function pJSON_PrintUnformatted(json : cJSON_ptr) : pstr;

var
   tmp : c_str_t;

begin
   tmp := cJSON_PrintUnformatted(json);
   pJSON_PrintUnformatted := pas_str(tmp);
   free_c_str(tmp);
end;

[external('CJSON_PRINT')]
function cJSON_Print(%IMMED json : cJSON_ptr) : c_str_t; extern;

function pJSON_Print(json : cJSON_ptr) : pstr;

var
   tmp : c_str_t;

begin
   tmp := cJSON_Print(json);
   pJSON_Print := pas_str(tmp);
   free_c_str(tmp);
end;

[external('CJSON_DELETE')]
procedure cJSON_Delete(%IMMED json : cJSON_ptr); extern;

procedure pJSON_Delete(json : cJSON_ptr);

begin
   cJSON_Delete(json);
end;

[external('CJSON_CREATEOBJECT')]
function cJSON_CreateObject : cJSON_ptr; extern;

function pJSON_CreateObject : cJSON_ptr;

begin
   pJSON_CreateObject := cJSON_CreateObject;
end;

[external('CJSON_CREATEARRAY')]
function cJSON_CreateArray : cJSON_ptr; extern;

function pJSON_CreateArray : cJSON_ptr;

begin
   pJSON_CreateArray := cJSON_CreateArray;
end;

[external('CJSON_CREATENULL')]
function cJSON_CreateNull : cJSON_ptr; extern;

function pJSON_CreateNull : cJSON_ptr;

begin
   pJSON_CreateNull := cJSON_CreateNull;
end;

[external('CJSON_CREATETRUE')]
function cJSON_CreateTrue : cJSON_ptr; extern;

function pJSON_CreateTrue : cJSON_ptr;

begin
   pJSON_CreateTrue := cJSON_CreateTrue;
end;

[external('CJSON_CREATEFALSE')]
function cJSON_CreateFalse : cJSON_ptr; extern;

function pJSON_CreateFalse : cJSON_ptr;

begin
   pJSON_CreateFalse := cJSON_CreateFalse;
end;

[external('CJSON_CREATENUMBER')]
function cJSON_CreateNumber(%IMMED v : double) : cJSON_ptr; extern;

function pJSON_CreateNumber(v : double) : cJSON_ptr;

begin
   pJSON_CreateNumber := cJSON_CreateNumber(v);
end;

[external('CJSON_CREATESTRING')]
function cJSON_CreateString(%IMMED s : c_str_t) : cJSON_ptr; extern;

function pJSON_CreateString(s : pstr) : cJSON_ptr;

var
   xs : c_str_t;

begin
   xs := malloc_c_str(s);
   pJSON_CreateString := cJSON_CreateString(xs);
   free_c_str(xs);
end;

[external('CJSON_CREATEBOOL')]
function cJSON_CreateBool(%IMMED v : integer) : cJSON_ptr; extern;

function pJSON_CreateBool(v : boolean) : cJSON_ptr;

begin
   if v then
      pJSON_CreateBool := cJSON_CreateBool(1)
   else
      pJSON_CreateBool := cJSON_CreateBool(0)
end;

[external('CJSON_ISFALSE')]
function cJSON_IsFalse(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsFalse(json : cJSON_ptr) : boolean;

begin
   pJSON_IsFalse := (cJSON_IsFalse(json) <> 0);
end;

[external('CJSON_ISTRUE')]
function cJSON_IsTrue(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsTrue(json : cJSON_ptr) : boolean;

begin
   pJSON_IsTrue := (cJSON_IsTrue(json) <> 0);
end;

[external('CJSON_ISBOOL')]
function cJSON_IsBool(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsBool(json : cJSON_ptr) : boolean;

begin
   pJSON_IsBool := (cJSON_IsBool(json) <> 0);
end;

[external('CJSON_ISNULL')]
function cJSON_IsNull(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsNull(json : cJSON_ptr) : boolean;

begin
   pJSON_IsNull := (cJSON_IsNull(json) <> 0);
end;

[external('CJSON_ISNUMBER')]
function cJSON_IsNumber(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsNumber(json : cJSON_ptr) : boolean;

begin
   pJSON_IsNumber := (cJSON_IsNumber(json) <> 0);
end;

[external('CJSON_ISSTRING')]
function cJSON_IsString(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsString(json : cJSON_ptr) : boolean;

begin
   pJSON_IsString := (cJSON_IsString(json) <> 0);
end;

[external('CJSON_ISARRAY')]
function cJSON_IsArray(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsArray(json : cJSON_ptr) : boolean;

begin
   pJSON_IsArray := (cJSON_IsArray(json) <> 0);
end;

[external('CJSON_ISOBJECT')]
function cJSON_IsObject(%IMMED json : cJSON_ptr) : integer; extern;

function pJSON_IsObject(json : cJSON_ptr) : boolean;

begin
   pJSON_IsObject := (cJSON_IsObject(json) <> 0);
end;

[external('CJSON_ADDITEMTOOBJECT')]
procedure cJSON_AddItemToObject(%IMMED json : cJSON_ptr; %IMMED propnam : c_str_t; %IMMED propval : cJSON_ptr); extern;

procedure pJSON_AddItemToObject(json : cJSON_ptr; propnam : pstr; propval : cJSON_ptr);

var
   xpropnam : c_str_t;

begin
   xpropnam := malloc_c_str(propnam);
   cJSON_AddItemToObject(json, xpropnam, propval);
   free_c_str(xpropnam);
end;

[external('CJSON_ADDITEMREFERENCETOOBJECT')]
procedure cJSON_AddItemReferenceToObject(%IMMED json : cJSON_ptr; %IMMED propnam : c_str_t; %IMMED propval : cJSON_ptr); extern;

procedure pJSON_AddItemReferenceToObject(json : cJSON_ptr; propnam : pstr; propval : cJSON_ptr);

var
   xpropnam : c_str_t;

begin
   xpropnam := malloc_c_str(propnam);
   cJSON_AddItemReferenceToObject(json, xpropnam, propval);
   free_c_str(xpropnam);
end;

[external('CJSON_ADDITEMTOARRAY')]
procedure cJSON_AddItemToArray(%IMMED json : cJSON_ptr; %IMMED elm : cJSON_ptr); extern;

procedure pJSON_AddItemToArray(json : cJSON_ptr; elm : cJSON_ptr);

begin
   cJSON_AddItemToArray(json, elm);
end;

[external('CJSON_ADDITEMREFERENCETOARRAY')]
procedure cJSON_AddItemReferenceToArray(%IMMED json : cJSON_ptr; %IMMED elm : cJSON_ptr); extern;

procedure pJSON_AddItemReferenceToArray(json : cJSON_ptr; elm : cJSON_ptr);

begin
   cJSON_AddItemReferenceToArray(json, elm);
end;

type
   trick = record
              case integer of
                 1 : (ia : integer_address);
                 2 : (ip : ^integer);
                 3 : (dp : ^double);
                 4 : (cs : ^c_str_t);
           end;

function pJSON_IntValue(json : cJSON_ptr) : integer;

var
   tmp : trick;

begin
   tmp.ia := json + 20;
   pJSON_IntValue := tmp.ip^;
end;

function pJSON_DoubleValue(json : cJSON_ptr) : double;

var
   tmp : trick;

begin
   tmp.ia := json + 24;
   pJSON_DoubleValue := tmp.dp^;
end;

function pJSON_StringValue(json : cJSON_ptr) : pstr;

var
   tmp : trick;

begin
   tmp.ia := json + 16;
   pJSON_StringValue := pas_str(tmp.cs^);
end;

function pJSON_BoolValue(json : cJSON_ptr) : boolean;

begin
   pJSON_BoolValue := (pJSON_IntValue(json) <> 0);
end;

end.
